// G. Greenleaf
// K. Nguyen
// The Breakout module

module breakout (clk, reset, en, enBall, l, r, ballX, ballY, paddleX, bricks, score, lives);

input clk, reset, l, r, en, enBall;
output [9:0] ballX, paddleX;
output [8:0] ballY;
output [59:0] bricks;
output [12:0] score;
output [1:0] lives;

reg [9:0] ballX, paddleX;
reg [8:0] ballY;
reg [59:0] bricks, newBricks;
reg [5:0] bricknum;
reg [12:0] score;
reg inBrick, newLevel, newLife;
reg [1:0] balldir, tmpDir, lives;

wire [9:0] tmpXa, tmpXb;
wire [8:0] tmpYa, tmpYb;
wire hittingBrick;
wire hitBrick;

assign hitBrick = (bricknum == 0) ? 0 : 1;
assign hittingBrick = (bricknum == 0 || bricknum > 60) ? hitBrick : bricks[bricknum-1];
 
inbrick moduleA(clk, tmpXa, tmpYa, paddleX, bricknum);
nextBallPos moduleB(clk, balldir, ballX, ballY, tmpXa, tmpYa);
nextBallPos moduleC(clk, tmpDir, ballX, ballY, tmpXb, tmpYb);
nextBallDir moduleD(clk, paddleX, balldir, ballX, ballY, tmpXa, tmpYa, tmpDir);

// calculate the paddle's next position
always @(posedge clk) begin
  if (en) begin
    if (reset)
      paddleX <= 320;
    else if (l && paddleX > 31)
      paddleX <= paddleX - 2;
    else if (r && paddleX < 609)
      paddleX <= paddleX + 2;
  end
end

// calculate the ball's next position, and the bricks
always @(posedge clk) begin
  if (reset) begin
    score <= 0;
    lives <= 3;
  end
  if (reset | newLevel) begin
    newLevel <= 0;
    bricks <= 60'hFFFFFFFFFFFFFFF;
  end
  if (reset | newLevel | newLife) begin
    newLife <= 0;
    ballX <= 320;
    ballY <= 456;
    balldir <= 0;
  end else if (enBall && lives != 0) begin
    if (ballY >= 478) begin
      lives <= lives - 1;
      newLife <= 1;
    end

    if (bricks == 0)
      newLevel <= 1;
      
    if ((hittingBrick & hitBrick) && (bricknum < 61)) begin
      if (bricknum > 50)
        score <= score + 1;
      else if (bricknum > 40)
        score <= score + 2;
      else if (bricknum > 30)
        score <= score + 3;
      else if (bricknum > 20)
        score <= score + 4;
      else if (bricknum > 10)
        score <= score + 5;
      else if (bricknum > 0)
        score <= score + 6;
        
      bricks[bricknum-1] <= 0;
    end
    
    if (hittingBrick) begin    
      ballX <= tmpXb;
      ballY <= tmpYb;
      balldir <= tmpDir;
    end else begin
      ballX <= tmpXa;
      ballY <= tmpYa;
    end
  end       
end

endmodule



module inbrick(clk, bx, by, paddleX, bricknum);

input clk;
input [9:0] bx, paddleX;
input [8:0] by;
output [5:0] bricknum;

wire [7:0] bnum;
wire [7:0] ynum;
reg [5:0] bricknum;

assign ynum = by[8:5];
assign bnum = (ynum >= 3 && ynum <= 8) ? 10 * ynum + bx[9:6] - 29 : 0;

always @(posedge clk) begin
  if ((by >= 458) && (by <= 460) && (bx > paddleX - 30) && (bx < paddleX + 30))
    bricknum <= 63;
  else if (bx > 638 || bx < 2 || by > 478 || by < 2)
    bricknum <= 62;
  else if (bnum > 60 || bnum==0) 
    bricknum <= 0;
  else
    bricknum <= bnum;
end


endmodule




module nextBallPos(clk, bdir, bx, by, outx, outy);

input clk;
input [1:0] bdir;
input [9:0] bx;
input [8:0] by;
output [9:0] outx;
output [8:0] outy;

reg [9:0] outx;
reg [8:0] outy;

always @(posedge clk) begin
  if (bdir[1])
    outx <= bx + 2;
  else
    outx <= bx - 2;
    
  if (bdir[0])
    outy <= by + 2;
  else
    outy <= by - 2;
end
endmodule



module nextBallDir(clk, paddleX, bdir, bx, by, tmpXa, tmpYa, outdir);
input clk;
input [1:0] bdir;
input [9:0] paddleX, bx, tmpXa;
input [8:0] by, tmpYa;
output [1:0] outdir; 

reg [1:0] outdir;

always @(posedge clk) begin
  if (bx < 2 || bx > 638)
    outdir[1] <= ~bdir[1];
  else if (((by < 70) || (by > 300 && by < 478)) || (((tmpXa[5:0] - bx[5:0]) <= 2) || ((bx[5:0] - tmpXa[5:0]) <= 2)))
    outdir[1] <= bdir[1];
  else
    outdir[1] <= ~bdir[1];
  
  if (by < 2 || by > 478)
    outdir[0] <= ~bdir[0];
  else if (bx < 2 || bx > 638)
    outdir[0] <= bdir[0];
  else if (((by < 458) || (by > 460) || (bx <= (paddleX - 30)) || (bx >= (paddleX + 30))) && (((by > 2 && by < 70) || (by > 300 && by < 478)) || (((tmpYa[4:0] - by[4:0]) <= 2) || ((by[4:0] - tmpYa[4:0]) <= 2)))) 
    outdir[0] <= bdir[0];
  else
    outdir[0] <= ~bdir[0];

end

endmodule


