module vga_hv_sync_unit(clk, reset, h_sync, v_sync, h_rgb, v_rgb);
   input clk, reset;
   output h_sync, v_sync;
   output h_rgb, v_rgb;

   h_sync_unit l_hsync(clk, reset, h_rgb, h_sync);
   v_sync_unit l_vsync(clk, reset, h_sync, v_rgb, v_sync);
   
endmodule // hv_sync


// Reimplementations of the H_SYNC, V_SYNC units in verilog
// Easy to do, great for testing

module h_sync_unit(clk, reset, h_rgb, h_sync);
   input clk, reset;
   output h_rgb, h_sync;
   reg 	  h_rgb, h_sync;

   reg [9:0] counter;
   
   always @(posedge clk) begin
      if(reset) begin
	 counter <= 0;
	 h_rgb <= 0;
	 h_sync <= 0;

      end else begin // if (reset)
	 counter <= counter + 1;

	 if(counter[6:0] == 7'b1011111)
	    h_sync <= 1;

	 if(counter[9:4] == 6'b110010) begin
	    counter <= 0;
	    h_sync <= 0;
	 end // if (counter[9:4] == 6'b010011)

	 if(counter[7:0] == 8'b10001000)
	    h_rgb <= 1;
	 if(counter[9:0] == 10'b1100001000)
	    h_rgb <= 0;
      end // else: !if(reset)
   end

endmodule // h_sync


// This v_sync unit might be more predictable than the original,
// because it's clocked by clk, not h_sync.

module v_sync_unit(clk, reset, h_sync, v_rgb, v_sync);
   input clk, reset, h_sync;
   output v_rgb, v_sync;
   reg 	  v_rgb, v_sync;

   reg 	  lh_sync;
   reg [9:0] counter;
   
   always @(posedge clk) begin
      if(reset) begin
	 counter <= 0;
	 v_rgb <= 0;
	 v_sync <= 0;

      end else begin

	 if(h_sync && !lh_sync) begin
	    counter <= counter + 1;

	    if(counter[0] && !v_sync)
	       v_sync <= 1;

	    if((counter[7:0] == 8'b00000010) && counter[9]) begin
	       counter <= 0;
	       v_sync <= 0;
	    end

	    if(counter[1] && counter[5] && !v_rgb)
	       v_rgb <= 1;
	    if((counter[7:0] == 8'b00000010) && counter[9])
	       v_rgb <= 0;
	 end 
	 lh_sync <= h_sync;

      end // else: !if(reset)
   end

endmodule // h_sync
